import 'dart:ffi';
import 'dart:io';
import 'package:ffi/ffi.dart';

///////////////////////////////////////////////////////////////////////////////
// C bindings
///////////////////////////////////////////////////////////////////////////////

// void rust_cstr_free(char *s);
// char *rust_greeting(const char *to);

///////////////////////////////////////////////////////////////////////////////
// Typedef's
///////////////////////////////////////////////////////////////////////////////

typedef ConvertFunc = Pointer<Utf8> Function(Pointer<Utf8>);
typedef FreeStringFunc = void Function(Pointer<Utf8>);
typedef FreeStringFuncNative = Void Function(Pointer<Utf8>);

///////////////////////////////////////////////////////////////////////////////
// Load the library
///////////////////////////////////////////////////////////////////////////////

final DynamicLibrary nativeExampleLib = Platform.isAndroid
    ? DynamicLibrary.open("libwaveform.so")
    : DynamicLibrary.process();

///////////////////////////////////////////////////////////////////////////////
// Locate the symbols we want to use
///////////////////////////////////////////////////////////////////////////////

final ConvertFunc convertFunc = nativeExampleLib
    .lookup<NativeFunction<ConvertFunc>>("convert_data")
    .asFunction();
final FreeStringFunc freeCString = nativeExampleLib
    .lookup<NativeFunction<FreeStringFuncNative>>("rust_cstr_free")
    .asFunction();
///////////////////////////////////////////////////////////////////////////////
// HANDLERS
///////////////////////////////////////////////////////////////////////////////

class WaveForm {
  static String callConvertData(String fileName) {
    print(fileName);
    if (nativeExampleLib == null)
      return "ERROR: The library is not initialized 🙁";
    if (fileName == "") {
      return "";
    }
    final argFileName = Utf8.toUtf8(fileName);
    final resultPointer = convertFunc(argFileName);
    String resultString;
    try {
      resultString = Utf8.fromUtf8(resultPointer);
    } catch (e) {
      return "";
    }

    freeCString(resultPointer);

    return resultString;
  }
}
