extern crate minimp3;

use minimp3::{Decoder, Error, Frame};
use std::os::raw::c_char;
use std::fs::File;
use std::ffi::{CStr, CString};

#[no_mangle]
pub extern "C" fn convert_data(ptr: *const c_char) -> *const c_char {
    let cstr = unsafe { CStr::from_ptr(ptr) };
    let wave_data: Vec<i16> = convert_to_wave_data_intern(cstr.to_str().unwrap());
    let wave_data_str: Vec<String> = wave_data.iter().map(|n| n.to_string()).collect();
    let joined_string: String = wave_data_str.join(",");
    let c_str = CString::new(joined_string).unwrap();
    return CString::into_raw(c_str);
}

fn convert_to_wave_data_intern(file_name: &str) -> Vec<i16> {
    let mut avs: Vec<i16> = Vec::new();

    let file = File::open(file_name);
    if file.is_err() {
        return avs;
    }
    let mut decoder = Decoder::new(file.unwrap());

    loop {
        match decoder.next_frame() {
            Ok(Frame {
                   data,
                   sample_rate: _sample_rate,
                   channels: _channels,
                   ..
               }) => {
                let mut mn = i16::max_value();
                let mut mx = i16::min_value();
                for sample in data {
                    mn = i16::min(sample, mn);
                    mx = i16::max(sample, mx);
                }
                if mn != 0 && mx != 0 {
                    avs.push(mn);
                    avs.push(mx);
                }
            }
            Err(_e) => break,
        }
    }
    drop(decoder);
    return avs;
}

#[no_mangle]
pub extern fn rust_cstr_free(s: *mut c_char) {
    unsafe {
        if s.is_null() { return; }
        CString::from_raw(s)
    };
}
